@@ -1,3 +1,19 @@
+#
+# This file is part of ruby-ffi.
+#
+# This code is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License version 3 only, as
+# published by the Free Software Foundation.
+#
+# This code is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# version 3 for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# version 3 along with this work.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 require File.expand_path(File.join(File.dirname(__FILE__), "spec_helper"))
 describe "Struct tests" do
   StructTypes = {
@@ -158,7 +174,7 @@ describe "Struct tests" do
       class TestStruct < FFI::Struct
         layout :c, :char
       end
-      attach_function :struct_field_s8, [ TestStruct ], :char
+      attach_function :struct_field_s8, [ TestStruct.in ], :char
     end
   end
   it "Can use Struct subclass as IN parameter type" do
@@ -306,11 +322,13 @@ describe "Struct tests" do
   end
   module EnumFields
     extend FFI::Library
-    enum :test_enum, [:c1, 10, :c2, 20, :c3, 30, :c4, 40]
+    TestEnum = enum :test_enum, [:c1, 10, :c2, 20, :c3, 30, :c4, 40]
     class TestStruct < FFI::Struct
-      layout :a, :int, :c, :test_enum
+      layout :a, :int, :c, :test_enum, 
+        :d, [ TestEnum, TestEnum.symbols.length ]
     end
   end
+  
   it ":enum field r/w" do
     s = EnumFields::TestStruct.new
     s[:c] = :c3
@@ -318,6 +336,22 @@ describe "Struct tests" do
     s.pointer.get_uint(FFI::Type::INT32.size).should == 30
     s[:c].should == :c3
   end
+  
+  it "array of :enum field" do
+    s = EnumFields::TestStruct.new
+    EnumFields::TestEnum.symbols.each_with_index do |val, i|
+      s[:d][i] = val
+    end
+    
+    EnumFields::TestEnum.symbols.each_with_index do |val, i|
+      s.pointer.get_uint(FFI::Type::INT32.size * (2 + i)).should == EnumFields::TestEnum[val]
+    end
+    
+    s[:d].each_with_index do |val, i|
+      val.should == EnumFields::TestEnum.symbols[i]
+    end
+  end
+  
   module CallbackMember
     extend FFI::Library
     ffi_lib TestLibrary::PATH
@@ -327,8 +361,8 @@ describe "Struct tests" do
       layout :add, :add,
         :sub, :sub
     end
-    attach_function :struct_call_add_cb, [TestStruct, :int, :int], :int
-    attach_function :struct_call_sub_cb, [TestStruct, :int, :int], :int
+    attach_function :struct_call_add_cb, [TestStruct.in, :int, :int], :int
+    attach_function :struct_call_sub_cb, [TestStruct.in, :int, :int], :int
   end
   it "Can have CallbackInfo struct field" do
       s = CallbackMember::TestStruct.new
@@ -336,8 +370,8 @@ describe "Struct tests" do
       sub_proc = lambda { |a, b| a-b }
       s[:add] = add_proc
       s[:sub] = sub_proc
-      CallbackMember.struct_call_add_cb(s.pointer, 40, 2).should == 42
-      CallbackMember.struct_call_sub_cb(s.pointer, 44, 2).should == 42
+      CallbackMember.struct_call_add_cb(s, 40, 2).should == 42
+      CallbackMember.struct_call_sub_cb(s, 44, 2).should == 42
   end
   it "Can return its members as a list" do
     class TestStruct < FFI::Struct
@@ -621,4 +655,18 @@ describe "Struct allocation" do
     p.put_uint(4, 0xdeadbeef)
     S.new(p[1])[:i].should == 0xdeadbeef
   end
+
+  it "null? should be true when initialized with NULL pointer" do
+    class S < FFI::Struct
+      layout :i, :uint
+    end
+    S.new(FFI::Pointer::NULL).null?.should be_true
+  end
+
+  it "null? should be false when initialized with non-NULL pointer" do
+    class S < FFI::Struct
+      layout :i, :uint
+    end
+    S.new(FFI::MemoryPointer.new(S)).null?.should be_false
+  end
 end
